package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.cassandra.thrift.Cassandra;
import org.apache.cassandra.thrift.Column;
import org.apache.cassandra.thrift.ColumnOrSuperColumn;
import org.apache.cassandra.thrift.ConsistencyLevel;
import org.apache.cassandra.thrift.InvalidRequestException;
import org.apache.cassandra.thrift.Mutation;
import org.apache.cassandra.thrift.TimedOutException;
import org.apache.cassandra.thrift.UnavailableException;
import org.apache.thrift.TException;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransportException;

/**
 * 東京都の郵便番号データをCSVからCassandraにインポートする.以下のURLからデータはダウンロードしてください.
 * 
 * @see http://www.post.japanpost.jp/zipcode/dl/kogaki/lzh/13tokyo.lzh
 */
public class SimpleAddressInsert {

	public static final String KEYSPACE = "Keyspace1";

	public static final String COLUMN_FAMILY = "SimpleAddress";

	public static void main(String[] args) {
		TSocket transport = new TSocket("localhost", 9160);
		TProtocol protocol = new TBinaryProtocol(transport);
		try {
			transport.open();
		} catch (TTransportException e) {
			throw new RuntimeException(e);
		}
		InputStream is = null;
		int count = 0;
		try {
			Cassandra.Client client = new Cassandra.Client(protocol);
			long timestamp = System.currentTimeMillis();

			Map<String, Map<String, List<Mutation>>> map = new HashMap<String, Map<String, List<Mutation>>>();

			is = Thread.currentThread().getContextClassLoader()
					.getResourceAsStream("13TOKYO.CSV");
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					is, "MS932"));
			String line = null;
			while ((line = reader.readLine()) != null) {
				List<Mutation> mutations = new ArrayList<Mutation>();
				String[] lines = line.split(",");
				String postalCode = removeQuote(lines[2]);
				String addressYomi = buildAddressYomi(lines);
				String address = buildAddress(lines);
				System.out.println(postalCode + " " + addressYomi + " "
						+ address);
				// 郵便番号をpostalCodeというカラムに設定する
				{
					Mutation mutation = new Mutation();
					ColumnOrSuperColumn csc = new ColumnOrSuperColumn();
					csc.setColumn(new Column("postalCode".getBytes(),
							postalCode.getBytes(), timestamp));
					mutation.setColumn_or_supercolumn(csc);
					mutations.add(mutation);
				}
				// 住所をaddressというカラムに設定する.
				{
					Mutation mutation = new Mutation();
					ColumnOrSuperColumn csc = new ColumnOrSuperColumn();
					csc.setColumn(new Column("address".getBytes(), address
							.getBytes(), timestamp));
					mutation.setColumn_or_supercolumn(csc);
					mutations.add(mutation);
				}
				// 住所の半角カナ読み方をaddressYomiというカラムに設定する.
				{
					Mutation mutation = new Mutation();
					ColumnOrSuperColumn csc = new ColumnOrSuperColumn();
					csc.setColumn(new Column("addressYomi".getBytes(),
							addressYomi.getBytes(), timestamp));
					mutation.setColumn_or_supercolumn(csc);
					mutations.add(mutation);
				}
				Map<String, List<Mutation>> mutmap = new HashMap<String, List<Mutation>>();
				mutmap.put(COLUMN_FAMILY, mutations);
				map.put(new String(postalCode), mutmap);
				count++;
			}
			client.batch_mutate(KEYSPACE, map, ConsistencyLevel.ONE);
			reader.close();
		} catch (InvalidRequestException e) {
			throw new RuntimeException(e);
		} catch (UnavailableException e) {
			throw new RuntimeException(e);
		} catch (TimedOutException e) {
			throw new RuntimeException(e);
		} catch (TException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			try {
				is.close();
			} catch (IOException ignore) {
				System.err.println(ignore.getMessage());
			}
			try {
				transport.flush();
			} catch (TTransportException e) {
				throw new RuntimeException(e);
			} finally {
				transport.close();
			}
		}
		System.out.println(count + "件インサート完了.");
	}

	/**
	 * 住所データから、半角カナの読み方部分を整形する.
	 * 
	 * @param lines
	 * @return 整形済みの半角カナ住所
	 */
	private static String buildAddressYomi(String[] lines) {
		StringBuilder builder = new StringBuilder();
		builder.append(removeQuote(lines[3]));
		builder.append(" ");
		builder.append(removeQuote(lines[4]));
		builder.append(" ");
		String s = removeQuote(lines[5]);
		builder.append(s.replace("ｲｶﾆｹｲｻｲｶﾞﾅｲﾊﾞｱｲ", ""));
		return new String(builder);
	}

	/**
	 * 住所データから、住所部分を整形する.
	 * 
	 * @param lines
	 * @return 整形済みの住所
	 */
	private static String buildAddress(String[] lines) {
		StringBuilder builder = new StringBuilder();
		builder.append(removeQuote(lines[6]));
		builder.append(" ");
		builder.append(removeQuote(lines[7]));
		builder.append(" ");
		String s = removeQuote(lines[8]);
		builder.append(s.replace("以下に掲載がない場合", ""));
		return new String(builder);
	}

	private static String removeQuote(String s) {
		return s.replaceAll("\"", "");
	}
}
